#include <signal.h>
#include <limits.h>

#include "wave.h"
#include "gsm/inc/gsm.h"
#include "dsp.h"

char *USAGE =

"USAGE: wav2gsm [options] [<wave-file>]\n"
"Converts 16 bit mono wave-file to GSM or records form DSP.\n"
"gsmrec is a link to wav2gsm and equivalent to 'wav2gsm -r'.\n"
"Options: -o<output file>, -h print this message, -v verbose mode\n"
"         -r record, -t<secs>s recording time (default: until SIGINT)\n"
"         -s<sample rate> in Hz, kHz or f (11025 Hz) (default: 8000Hz)\n";

gsm handle;
gsm_frame buf;
gsm_signal sample[160];

volatile int term_req=0;

void sighandler(int sig) {
  term_req=1;
} 


int main(int argc,char **argv)
{
  wavfile w,g;
  LONGINT i,j;
  char c;
  int record=0;
  char *env;
  REAL t=(-1.),x;
  int dspfn;

  setexitmsg(argv[0]);
  g.f=stdout;
  w.rate=8000;

  env=getenv("GSM_RATE");
  if(env && atoi(env)>0) w.rate=atoi(env);

  i=strlen(argv[0]);
  if(i>=6 && strncasecmp(argv[0]+i-6,"gsmrec",6)==0) record=1;

  while(1)
  {
    c=getopt(argc,argv,"o:hvt:s:r");
    if(stdopt(c,USAGE)) break;
    switch(c)
    {
      case 'o': g.f=fileopen(optarg,"wb",0); break;
      case 's':
        x=getfreq(optarg);
        if(x<=0)
        {
          fprintf(stderr,"%s: Invalid sampling rate\n",PRG);
          errexit(1);
        };
        w.rate=rint(x);
        break;
      case 'r': record=1; break;
      case 't':
	t=gettime(optarg); break;
    };
  };
  if(optind<argc-1) errexit(ERR_ARG);
  
  if(!record) {
    if(optind<argc)
      wavfin(&w,argv[optind]);
    else
      wavstdin(&w);

    if(w.bits!=16) {
      fprintf(stderr,"%s: Input file is not 16 bit.\n",PRG);
      errexit(ERR_FORMAT);
    }
  } else {
    if(t>0) {
      w.n=160*((w.rate*t+159)/160);
    } else {
      w.n=INT_MAX;
    }
    dspfn=opendsp(16,w.rate,0,O_RDONLY);
    w.i=0;
  }

  if (!(handle = gsm_create())) exit(1);

  signal(SIGINT,&sighandler);

  while (w.i<w.n && !term_req) {
    if(record) {
      w.i+=read(dspfn,sample,320)/2;
    } else {
      for(j=0;j<160;j++) sample[j]= (w.i<w.n) ? wavbinget(&w) : 0;
    }
    gsm_encode(handle, sample, buf);
    fwrite((char *)buf, sizeof(buf),1,g.f);
  }   
  if(record) close(dspfn);
  fclose(g.f);
}

